@@ -9,7 +9,129 @@ local ($empty) = @_;
 my @rv;
 my $ethtool = &has_command("ethtool");
 
-if (&has_command("ifconfig")) {
+if (&has_command("ip")) {
+	# Get status from new ip command
+	&clean_language();
+	&open_execute_command(IFC, "ip addr", 1, 1);
+	my @lines;
+	while(<IFC>) {
+		s/\r|\n//g;
+		if (/^\S+/) { push(@lines, $_); }
+		else { $lines[$#lines] .= $_; }
+		}
+	close(IFC);
+	
+	&reset_environment();
+	foreach my $l (@lines) {
+		my %ifc;
+		if ($l =~ /^\d+:\s+([^ \t\r\n\@]+\d+\.(\d+))@([^ \t\r\n\@]+\d+):/) {
+			# Line like :
+			# 3: eth0.99@eth0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue state UP group default qlen 1000
+			$ifc{'fullname'} = $1;
+			$ifc{'name'} = $ifc{'fullname'};
+			$ifc{'vlanid'} = $2;
+			$ifc{'virtual'} = $3;
+			}
+		elsif ($l =~ /^\d+:\s+([^ \t\r\n\@]+):/) {
+			# Line like :
+			# 2: eth0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast state UP group default qlen 1000
+			$ifc{'fullname'} = $1;
+			$ifc{'name'} = $ifc{'fullname'}
+			}
+		elsif ($l =~ /^\d:\s+([^ \t\r\n\@]+\d+)@([^ \t\r\n\@]+\d+):/) {
+			# Line like :
+			# 3: eth0@if0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue state UP group default qlen 1000
+			$ifc{'fullname'} = $1;
+			$ifc{'name'} = $ifc{'fullname'};
+			$ifc{'ifname'} = $2;
+			}
+		else {
+			# Unknown line!
+			next;
+			}
+		
+		if ($l =~ /\sinet\s+([0-9\.]+)\s+peer\s+([0-9\.]+)\/(\d+)(\s+brd\s+([0-9\.]+))?\s+scope\s+global(\s+(noprefixroute|dynamic))*\s+(\S+)/ && $8 eq $ifc{'name'}) {
+			# Line like :
+			# inet 193.9.101.120 peer 193.9.101.104/32 brd 193.9.101.120 scope global eth0
+			$ifc{'address'} = $1;
+			$ifc{'netmask'} = &prefix_to_mask("$3");
+			}
+		elsif ($l =~ /\sinet\s+([0-9\.]+)\/(\d+)(\s+brd\s+(\S+))?\s+scope\s+global(\s+(noprefixroute|dynamic))*\s+(\S+)/ && $7 eq $ifc{'name'}) {
+			# Line like :
+			# inet 193.9.101.120/24 brd 193.9.101.255 scope global br0
+			$ifc{'address'} = $1;
+			$ifc{'netmask'} = &prefix_to_mask("$2");
+			}
+		elsif ($l =~ /\sinet\s+([0-9\.]+)\s+peer\s+([0-9\.]+)\/(\d+)/) {
+			# Line like :
+			# inet 46.4.13.87 peer 46.4.13.65/32
+			$ifc{'address'} = $1;
+			$ifc{'netmask'} = &prefix_to_mask("$3");
+			}
+		elsif ($l =~ /\sinet\s+([0-9\.]+)\/(\d+)(\s+brd\s+(\S+))?\s+scope\s+host\s+(\S+)/ && $5 eq $ifc{'name'}) {
+			# Line like :
+			# inet 127.0.0.1/8 scope host lo
+                        $ifc{'address'} = $1;
+                        $ifc{'netmask'} = &prefix_to_mask("$2");
+			}
+		if ($l =~ /\sbrd\s+([0-9\.]+)/) {
+			$ifc{'broadcast'} = $1;
+			}
+		if ($l =~ /\smtu\s+(\d+)/) {
+			$ifc{'mtu'} = $1;
+			}
+		if ($l =~ /ether\s+([0-9a-f:]+)/i) {
+			$ifc{'ether'} = $1;
+			}
+
+		my (@address6, @netmask6, @scope6);
+		while($l =~ s/inet6\s+(\S+)\/(\d+)\s+scope\s+(\S+)//i) {
+			local ($address6, $netmask6, $scope6) = ($1, $2, $3);
+			push(@address6, $address6);
+			push(@netmask6, $netmask6);
+			push(@scope6, $scope6);
+			}
+		$ifc{'address6'} = \@address6;
+		$ifc{'netmask6'} = \@netmask6;
+		$ifc{'scope6'} = \@scope6;
+
+		$ifc{'up'}++ if ($l =~ /\sUP\s|<\S*UP\S*>/);
+		$ifc{'promisc'}++ if ($l =~ /\sPROMISC\s/);
+		$ifc{'edit'} = ($ifc{'name'} !~ /^ppp/);
+		$ifc{'index'} = scalar(@rv);
+		push(@rv, \%ifc);
+
+		# Strip off the line for the primary IP, but only if there
+		# is one
+		if ($ifc{'address'}) {
+			$l =~ s/\sinet\s+([0-9\.]+)\s+peer// ||
+				$l =~ s/\sinet\s+([0-9\.]+)\/(\d+)//;
+			}
+
+		# Add extra IPs as fake virtual interfaces, from lines like
+		my $i = 0;
+		my $bn = $ifc{'name'};
+		while($l =~ s/\sinet\s+([0-9\.]+)\/(\d+).*?\s\Q$bn\E:(\d+)// ||
+		      $l =~ s/\sinet\s+([0-9\.]+)\/(\d+).*?secondary.*?\s\Q$bn\E//) {
+			my %vifc;
+			my $vn = defined($3) ? $3 : $i;
+			$vifc{'name'} = $ifc{'name'};
+			$vifc{'fullname'} = $ifc{'name'}.":".$vn;
+			$vifc{'address'} = $1;
+			$vifc{'netmask'} = &prefix_to_mask("$2");
+			$vifc{'broadcast'} = &compute_broadcast(
+				$vifc{'address'}, $vifc{'netmask'});
+			$vifc{'mtu'} = $ifc{'mtu'};
+			$vifc{'up'} = $ifc{'up'};
+			$vifc{'virtual'} = $vn;
+			$vifc{'edit'} = ($vifc{'name'} !~ /^ppp/);
+			$vifc{'index'} = scalar(@rv);
+			push(@rv, \%vifc);
+			$i++;
+			}
+		}
+	}
+elsif (&has_command("ifconfig")) {
 	&clean_language();
 	&open_execute_command(IFC, "ifconfig -a", 1, 1);
 	my @lines;
@@ -67,79 +189,7 @@ if (&has_command("ifconfig")) {
 		push(@rv, \%ifc);
 		}
 	}
-elsif (&has_command("ip")) {
-	# Get status from new ip command
-	&clean_language();
-	&open_execute_command(IFC, "ip addr", 1, 1);
-	my @lines;
-	while(<IFC>) {
-		s/\r|\n//g;
-		if (/^\S+/) { push(@lines, $_); }
-		else { $lines[$#lines] .= $_; }
-		}
-	close(IFC);
-	&reset_environment();
-	foreach my $l (@lines) {
-		my %ifc;
-		$l =~ /^\d+:\s+(\S+):/ || next;
-		$ifc{'name'} = $1;
-		$ifc{'fullname'} = $1;
-		if ($l =~ /\sinet\s+([0-9\.]+)\/(\d+)/) {
-			$ifc{'address'} = $1;
-			$ifc{'netmask'} = &prefix_to_mask("$2");
-			}
-		elsif ($l =~ /\sinet\s+([0-9\.]+)\s+peer\s+([0-9\.]+)\/(\d+)/) {
-			$ifc{'address'} = $1;
-			$ifc{'netmask'} = &prefix_to_mask("$3");
-			}
-		if ($l =~ /\sbrd\s+([0-9\.]+)/) {
-			$ifc{'broadcast'} = $1;
-			}
-		if ($l =~ /\smtu\s+(\d+)/) {
-			$ifc{'mtu'} = $1;
-			}
-		if ($l =~ /ether\s+([0-9a-f:]+)/i) {
-			$ifc{'ether'} = $1;
-			}
 
-		my (@address6, @netmask6, @scope6);
-		while($l =~ s/inet6\s+(\S+)\/(\d+)\s+scope\s+(\S+)//i) {
-			local ($address6, $netmask6, $scope6) = ($1, $2, $3);
-			push(@address6, $address6);
-			push(@netmask6, $netmask6);
-			push(@scope6, $scope6);
-			}
-		$ifc{'address6'} = \@address6;
-		$ifc{'netmask6'} = \@netmask6;
-		$ifc{'scope6'} = \@scope6;
-
-		$ifc{'up'}++ if ($l =~ /\sUP\s|<\S*UP\S*>/);
-		$ifc{'promisc'}++ if ($l =~ /\sPROMISC\s/);
-		$ifc{'edit'} = ($ifc{'name'} !~ /^ppp/);
-		$ifc{'index'} = scalar(@rv);
-		push(@rv, \%ifc);
-
-		# Add extra IPs as fake virtual interfaces
-		$l =~ s/\sinet\s+([0-9\.]+)\/(\d+)//;
-		my $i = 0;
-		while($l =~ s/\sinet\s+([0-9\.]+)\/(\d+)//) {
-			my %vifc;
-			$vifc{'name'} = $ifc{'name'};
-			$vifc{'fullname'} = $ifc{'name'}.":".$i;
-			$vifc{'address'} = $1;
-			$vifc{'netmask'} = &prefix_to_mask("$2");
-			$vifc{'broadcast'} = &compute_broadcast(
-				$vifc{'address'}, $vifc{'netmask'});
-			$vifc{'mtu'} = $ifc{'mtu'};
-			$vifc{'up'} = $ifc{'up'};
-			$vifc{'virtual'} = $i;
-			$vifc{'edit'} = ($vifc{'name'} !~ /^ppp/);
-			$vifc{'index'} = scalar(@rv);
-			push(@rv, \%vifc);
-			$i++;
-			}
-		}
-	}
 else {
 	&error("Both the ifconfig and ip commands are missing");
 	}
@@ -150,10 +200,10 @@ foreach my $ifc (@rv) {
 	    $ifc->{'virtual'} eq '' && $ethtool) {
 		my $out = &backquote_command(
 			"$ethtool $ifc->{'fullname'} 2>/dev/null");
-		if ($out =~ /Speed:\s+(\S+)/i) {
+		if ($out =~ /Speed:\s+(\S+)/i && $1 ne "Unknown!") {
 			$ifc->{'speed'} = $1;
 			}
-		if ($out =~ /Duplex:\s+(\S+)/i) {
+		if ($out =~ /Duplex:\s+(\S+)/i && $1 ne "Unknown!") {
 			$ifc->{'duplex'} = $1;
 			}
 		if ($out =~ /Link\s+detected:\s+(\S+)/i) {
@@ -460,7 +510,7 @@ return "VmWare" if ($name =~ /^vmnet/);
 return "Wireless" if ($name =~ /^wlan/);
 return "Bonded" if ($name =~ /^bond/);
 return "OpenVZ" if ($name =~ /^venet/);
-return "Bridge" if ($name =~ /^br/);
+return "Bridge" if ($name =~ /^(br|xenbr|virbr)/);
 return $text{'ifcs_unknown'};
 }
 
@@ -474,7 +524,7 @@ if (&has_command("netstat")) {
 	&open_execute_command(ROUTES, "netstat -rn", 1, 1);
 	while(<ROUTES>) {
 		s/\s+$//;
-		if (/^([0-9\.]+)\s+([0-9\.]+)\s+([0-9\.]+)\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)$/) {
+		if (/^([0-9\.]+)\s+([0-9\.]+)\s+([0-9\.]+)\s+\S+\s+\S+\s+\S+\s*\S+\s+(\S+)$/) {
 			push(@rv, { 'dest' => $1,
 				    'gateway' => $2,
 				    'netmask' => $3,
@@ -485,7 +535,7 @@ if (&has_command("netstat")) {
 	&open_execute_command(ROUTES, "netstat -rn -A inet6", 1, 1);
 	while(<ROUTES>) {
 		s/\s+$//;
-		if (/^([0-9a-z:]+)\/([0-9]+)\s+([0-9a-z:]+)\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)$/) {
+		if (/^([0-9a-z:]+)\/([0-9]+)\s+([0-9a-z:]+)\s+\S+\s+\S+\s+\S+\s*\S+\s+(\S+)$/) {
 			push(@rv, { 'dest' => $1,
 				    'gateway' => $3,
 				    'netmask' => $2,
@@ -714,10 +764,12 @@ return 0;
 
 # get_dns_config()
 # Returns a hashtable containing keys nameserver, domain, search & order
+# XXX factor out to os_ functions
 sub get_dns_config
 {
 local $dns = { };
 local $rc;
+local $dnsfile;
 if ($use_suse_dns && ($rc = &parse_rc_config()) && $rc->{'NAMESERVER'}) {
 	# Special case - get DNS settings from SuSE config
 	local @ns = split(/\s+/, $rc->{'NAMESERVER'}->{'value'});
@@ -726,8 +778,20 @@ if ($use_suse_dns && ($rc = &parse_rc_config()) && $rc->{'NAMESERVER'}) {
 	$dns->{'domain'} = [ split(/\s+/, $src->{'value'}) ] if ($src);
 	$dnsfile = $rc_config;
 	}
-elsif ($gconfig{'os_type'} eq 'debian-linux' &&
-       -l "/etc/resolv.conf") {
+elsif ($gconfig{'os_type'} eq 'debian-linux' && -l "/etc/resolv.conf" &&
+       $netplan_dir) {
+	# On Ubuntu 18+, /etc/resolv.conf is auto-generated from netplan config
+	my @boot = &boot_interfaces();
+	foreach my $b (@boot) {
+		if ($b->{'nameserver'}) {
+			$dns->{'nameserver'} = $b->{'nameserver'};
+			$dns->{'domain'} = $b->{'search'};
+			$dnsfile = $b->{'file'};
+			last;
+			}
+		}
+	}
+elsif ($gconfig{'os_type'} eq 'debian-linux' && -l "/etc/resolv.conf") {
 	# On Ubuntu 12+, /etc/resolv.conf is auto-generated from network
 	# interface config
 	my @ifaces = &get_interface_defs();
